@@ -1,9 +1,9 @@
-/*	$calcurse: calcurse.c,v 1.85 2009-07-26 20:38:35 culot Exp $	*/
+/*	$calcurse: calcurse.c,v 1.88 2010-03-20 13:29:47 culot Exp $	*/
 
 /*
  * Calcurse - text-based organizer
  *
- * Copyright (c) 2004-2009 Frederic Culot <frederic@culot.org>
+ * Copyright (c) 2004-2010 Frederic Culot <frederic@culot.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -38,22 +38,7 @@
 
 #include <stdlib.h>
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif /* HAVE_CONFIG_H */
-
-#include "i18n.h"
-#include "io.h"
-#include "help.h"
-#include "custom.h"
-#include "utils.h"
-#include "sigs.h"
-#include "day.h"
-#include "todo.h"
-#include "args.h"
-#include "notify.h"
-#include "keys.h"
-#include "dmon.h"
+#include "calcurse.h"
 
 /*
  * Calcurse is a text-based personal organizer which helps keeping track
@@ -65,8 +50,8 @@
 int
 main (int argc, char **argv)
 {
-  conf_t conf;
-  day_items_nb_t inday;
+  struct conf conf;
+  struct day_items_nb inday;
   int background, foreground;
   int non_interactive;
   int no_data_file = 1;
@@ -294,6 +279,10 @@ main (int argc, char **argv)
                 case 'k':
                   custom_keys_config ();
                   break;
+                case 's':
+                case 'S':
+                  custom_sidebar_config ();
+                  break;
 		}
 	      wins_reset ();
 	      wins_update ();
@@ -526,6 +515,16 @@ main (int argc, char **argv)
 	      calendar_move (WEEK_END);
 	    }
           break;
+
+        case KEY_GENERIC_SCROLL_UP:
+          if (wins_slctd () == CAL)
+            calendar_view_prev ();
+          break;
+
+        case KEY_GENERIC_SCROLL_DOWN:
+          if (wins_slctd () == CAL)
+            calendar_view_next ();
+          break;
           
         case KEY_GENERIC_QUIT:          
 	  if (conf.auto_save)
