@@ -1,9 +1,9 @@
-/*	$calcurse: calendar.c,v 1.25 2009-07-12 16:21:59 culot Exp $	*/
+/*	$calcurse: calendar.c,v 1.37 2010-03-29 11:48:05 culot Exp $	*/
 
 /*
  * Calcurse - text-based organizer
  *
- * Copyright (c) 2004-2009 Frederic Culot <frederic@culot.org>
+ * Copyright (c) 2004-2010 Frederic Culot <frederic@culot.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -43,15 +43,7 @@
 #include <time.h>
 #include <math.h>
 
-#include "i18n.h"
-#include "day.h"
-#include "apoint.h"
-#include "event.h"
-#include "custom.h"
-#include "vars.h"
-#include "keys.h"
-#include "utils.h"
-#include "calendar.h"
+#include "calcurse.h"
 
 #ifndef M_PI
 #define	M_PI	  3.14159265358979323846
@@ -64,15 +56,63 @@
 #define	lzero	  318.351648	/* lunar mean long at EPOCH */
 #define	Pzero	  36.340410	/* lunar mean long of perigee at EPOCH */
 #define	Nzero	  318.510107	/* lunar mean long of node at EPOCH */
-#define TM_YEAR_BASE	1900
 
 #define isleap(y) ((((y) % 4) == 0 && ((y) % 100) != 0) || ((y) % 400) == 0)
 
-static date_t today, slctd_day;
-static unsigned week_begins_on_monday;
+enum {
+  CAL_MONTH_VIEW,
+  CAL_WEEK_VIEW,
+  CAL_VIEWS
+};
+
+enum pom {
+  NO_POM,
+  FIRST_QUARTER,
+  FULL_MOON,
+  LAST_QUARTER,
+  NEW_MOON,
+  MOON_PHASES
+};
+
+static struct date today, slctd_day;
+static unsigned calendar_view, week_begins_on_monday;
 static pthread_mutex_t date_thread_mutex = PTHREAD_MUTEX_INITIALIZER;
 static pthread_t calendar_t_date;
 
+static void draw_monthly_view (struct window *, struct date *, unsigned);
+static void draw_weekly_view (struct window *, struct date *, unsigned);
+static void (*draw_calendar[CAL_VIEWS]) (struct window *, struct date *,
+                                         unsigned) =
+  {draw_monthly_view, draw_weekly_view};
+
+/* Switch between calendar views (monthly view is selected by default). */
+void
+calendar_view_next (void)
+{
+  calendar_view++;
+  if (calendar_view == CAL_VIEWS)
+    calendar_view = 0;
+}
+
+void
+calendar_view_prev (void)
+{
+  if (calendar_view == 0)
+    calendar_view = CAL_VIEWS;
+  calendar_view--;
+}
+
+void
+calendar_set_view (int view)
+{
+  calendar_view = (view < 0 || view >= CAL_VIEWS) ? CAL_MONTH_VIEW : view;
+}
+
+int
+calendar_get_view (void)
+{
+  return (int)calendar_view;
+}
 
 /* Thread needed to update current date in calendar. */
 /* ARGSUSED0 */
@@ -89,8 +129,10 @@ calendar_date_thread (void *arg)
         (void)sleep (tomorrow - actual);
 
       calendar_set_current_date ();
-      calendar_update_panel (win[CAL].p);
+      calendar_update_panel (&win[CAL]);
     }
+
+  return (void *)0;
 }
 
 /* Launch the calendar date thread. */
@@ -127,7 +169,7 @@ calendar_set_current_date (void)
 
 /* Needed to display sunday or monday as the first day of week in calendar. */
 void
-calendar_set_first_day_of_week (wday_e first_day)
+calendar_set_first_day_of_week (enum wday first_day)
 {
   switch (first_day)
     {
@@ -160,7 +202,7 @@ calendar_week_begins_on_monday (void)
 
 /* Fill in the given variable with the current date. */
 void
-calendar_store_current_date (date_t *date)
+calendar_store_current_date (struct date *date)
 {
   pthread_mutex_lock (&date_thread_mutex);
   *date = today;
@@ -175,7 +217,7 @@ calendar_init_slctd_day (void)
 }
 
 /* Return the selected day in calendar */
-date_t *
+struct date *
 calendar_get_slctd_day (void)
 {
   return (&slctd_day);
@@ -188,6 +230,21 @@ calendar_get_slctd_day_sec (void)
   return (date2sec (slctd_day, 0, 0));
 }
 
+static int
+calendar_get_wday (struct date *date)
+{
+  struct tm t;
+
+  (void)memset (&t, 0, sizeof (struct tm));
+  t.tm_mday = date->dd;
+  t.tm_mon = date->mm - 1;
+  t.tm_year = date->yyyy - 1900;
+
+  (void)mktime (&t);
+
+  return t.tm_wday;
+}
+
 static int
 isBissextile (unsigned year)
 {
@@ -221,29 +278,48 @@ ymd_to_scalar (unsigned year, unsigned month, unsigned day)
   return (scalar);
 }
 
-/* Function used to display the calendar panel. */
-void
-calendar_update_panel (WINDOW *cwin)
+/* 
+ * Used to change date by adding a certain amount of days or weeks.
+ * Returns 0 on success, 1 otherwise.
+ */
+static int
+date_change (struct tm *date, int delta_month, int delta_day)
 {
-  date_t current_day, check_day;
+  struct tm t;
+
+  t = *date;
+  t.tm_mon += delta_month;
+  t.tm_mday += delta_day;
+
+  if (mktime (&t) == -1)
+    return (1);
+  else
+    {
+      *date = t;
+      return (0);
+    }
+}
+
+/* Draw the monthly view inside calendar panel. */
+static void
+draw_monthly_view (struct window *cwin, struct date *current_day,
+                   unsigned sunday_first)
+{
+  const int OFFY = 2 + (CALHEIGHT - 9) / 2;
+  struct date check_day;
   int c_day, c_day_1, day_1_sav, numdays, j;
   unsigned yr, mo;
-  int ofs_x, ofs_y;
+  int OFFX, SBAR_WIDTH, ofs_x, ofs_y;
   int item_this_day = 0;
-  int title_lines = 3;
-  int sunday_first = 0;
 
-  /* inits */
-  calendar_store_current_date (&current_day);
-  erase_window_part (cwin, 1, title_lines, CALWIDTH - 2, CALHEIGHT - 2);
   mo = slctd_day.mm;
   yr = slctd_day.yyyy;
-  if (!calendar_week_begins_on_monday ())
-    sunday_first = 1;
 
   /* offset for centering calendar in window */
-  ofs_y = 2 + (CALHEIGHT - 9) / 2;
-  ofs_x = (CALWIDTH - 27) / 2;
+  SBAR_WIDTH = wins_sbar_width ();
+  OFFX = (SBAR_WIDTH - 27) / 2;
+  ofs_y = OFFY;
+  ofs_x = OFFX;
 
   /* checking the number of days in february */
   numdays = days[mo - 1];
@@ -257,20 +333,21 @@ calendar_update_panel (WINDOW *cwin)
   c_day_1 = (int) ((ymd_to_scalar (yr, mo, 1 + sunday_first) - (long) 1) % 7L);
 
   /* Write the current month and year on top of the calendar */
-  custom_apply_attr (cwin, ATTR_HIGHEST);
-  mvwprintw (cwin, ofs_y, (CALWIDTH - (strlen (_(monthnames[mo - 1])) + 5)) / 2,
+  custom_apply_attr (cwin->p, ATTR_HIGHEST);
+  mvwprintw (cwin->p, ofs_y,
+             (SBAR_WIDTH - (strlen (_(monthnames[mo - 1])) + 5)) / 2,
 	     "%s %d", _(monthnames[mo - 1]), slctd_day.yyyy);
-  custom_remove_attr (cwin, ATTR_HIGHEST);
+  custom_remove_attr (cwin->p, ATTR_HIGHEST);
   ++ofs_y;
 
   /* print the days, with regards to the first day of the week */
-  custom_apply_attr (cwin, ATTR_HIGHEST);
-  for (j = 0; j < 7; j++)
+  custom_apply_attr (cwin->p, ATTR_HIGHEST);
+  for (j = 0; j < WEEKINDAYS; j++)
     {
-      mvwprintw (cwin, ofs_y, ofs_x + 4 * j, "%s",
+      mvwprintw (cwin->p, ofs_y, ofs_x + 4 * j, "%s",
 		 _(daynames[1 + j - sunday_first]));
     }
-  custom_remove_attr (cwin, ATTR_HIGHEST);
+  custom_remove_attr (cwin->p, ATTR_HIGHEST);
 
   day_1_sav = (c_day_1 + 1) * 3 + c_day_1 - 7;
 
@@ -286,61 +363,255 @@ calendar_update_panel (WINDOW *cwin)
       /* Go to next line, the week is over. */
       if (!c_day_1 && 1 != c_day)
 	{
-	  ++ofs_y;
-	  ofs_x = 2 - day_1_sav - 4 * c_day - 1;
+	  ofs_y++;
+          ofs_x = OFFX - day_1_sav - 4 * c_day;
 	}
 
       /* This is today, so print it in yellow. */
-      if (c_day == current_day.dd && current_day.mm == slctd_day.mm
-	  && current_day.yyyy == slctd_day.yyyy
-	  && current_day.dd != slctd_day.dd)
+      if (c_day == current_day->dd
+          && current_day->mm == slctd_day.mm
+          && current_day->yyyy == slctd_day.yyyy
+          && current_day->dd != slctd_day.dd)
 	{
-	  custom_apply_attr (cwin, ATTR_LOWEST);
-	  mvwprintw (cwin, ofs_y + 1,
-		     ofs_x + day_1_sav + 4 * c_day + 1, "%2d", c_day);
-	  custom_remove_attr (cwin, ATTR_LOWEST);
+	  custom_apply_attr (cwin->p, ATTR_LOWEST);
+	  mvwprintw (cwin->p, ofs_y + 1, 
+                     ofs_x + day_1_sav + 4 * c_day + 1, "%2d", c_day);
+	  custom_remove_attr (cwin->p, ATTR_LOWEST);
 	}
-      else if (c_day == slctd_day.dd &&
-	       ((current_day.dd != slctd_day.dd)
-		|| (current_day.mm != slctd_day.mm)
-		|| (current_day.yyyy != slctd_day.yyyy)))
+      else if (c_day == slctd_day.dd)
 	{
 	  /* This is the selected day, print it according to user's theme. */
-	  custom_apply_attr (cwin, ATTR_HIGHEST);
-	  mvwprintw (cwin, ofs_y + 1, ofs_x + day_1_sav + 4 * c_day + 1, "%2d",
+	  custom_apply_attr (cwin->p, ATTR_HIGHEST);
+	  mvwprintw (cwin->p, ofs_y + 1, 
+                     ofs_x + day_1_sav + 4 * c_day + 1, "%2d",
                      c_day);
-	  custom_remove_attr (cwin, ATTR_HIGHEST);
-	}
-      else if (c_day == slctd_day.dd && current_day.dd == slctd_day.dd
-	       && current_day.mm == slctd_day.mm
-	       && current_day.yyyy == slctd_day.yyyy)
-	{
-	  /* today is the selected day */
-	  custom_apply_attr (cwin, ATTR_HIGHEST);
-	  mvwprintw (cwin, ofs_y + 1, ofs_x + day_1_sav + 4 * c_day + 1, "%2d",
-                     c_day);
-	  custom_remove_attr (cwin, ATTR_HIGHEST);
+	  custom_remove_attr (cwin->p, ATTR_HIGHEST);
 	}
       else if (item_this_day)
 	{
-	  custom_apply_attr (cwin, ATTR_LOW);
-	  mvwprintw (cwin, ofs_y + 1, ofs_x + day_1_sav + 4 * c_day + 1, "%2d",
+	  custom_apply_attr (cwin->p, ATTR_LOW);
+	  mvwprintw (cwin->p, ofs_y + 1, 
+                     ofs_x + day_1_sav + 4 * c_day + 1, "%2d",
                      c_day);
-	  custom_remove_attr (cwin, ATTR_LOW);
+	  custom_remove_attr (cwin->p, ATTR_LOW);
 	}
       else
 	/* otherwise, print normal days in black */
-	mvwprintw (cwin, ofs_y + 1, ofs_x + day_1_sav + 4 * c_day + 1, "%2d",
+	mvwprintw (cwin->p, ofs_y + 1,
+                   ofs_x + day_1_sav + 4 * c_day + 1, "%2d",
                    c_day);
     }
-  wnoutrefresh (cwin);
+}
+
+static int
+weeknum (const struct tm *t, int firstweekday)
+{
+  int wday, wnum;
+
+  wday = t->tm_wday;
+  if (firstweekday == MONDAY)
+    {
+      if (wday == SUNDAY)
+        wday = 6;
+      else
+        wday--;
+    }
+  wnum = ((t->tm_yday + WEEKINDAYS - wday) / WEEKINDAYS);
+  if (wnum < 0)
+    wnum = 0;
+  
+  return wnum;
+}
+
+/*
+ * Compute the week number according to ISO 8601.
+ */
+static int
+ISO8601weeknum (const struct tm *t)
+{
+  int wnum, jan1day;
+
+  wnum = weeknum (t, MONDAY);
+
+  jan1day = t->tm_wday - (t->tm_yday % WEEKINDAYS);
+  if (jan1day < 0)
+    jan1day += WEEKINDAYS;
+
+  switch (jan1day)
+    {
+    case MONDAY:
+      break;
+    case TUESDAY:
+    case WEDNESDAY:
+    case THURSDAY:
+      wnum++;
+      break;
+    case FRIDAY:
+    case SATURDAY:
+    case SUNDAY:
+      if (wnum == 0)
+        {
+          /* Get week number of last week of last year. */
+          struct tm dec31ly; /* 12/31 last year */
+          
+          dec31ly = *t;
+          dec31ly.tm_year--;
+          dec31ly.tm_mon = 11;
+          dec31ly.tm_mday = 31;
+          dec31ly.tm_wday = (jan1day == SUNDAY) ? 6 : jan1day - 1;
+          dec31ly.tm_yday = 364 + isBissextile (dec31ly.tm_year + 1900);
+          wnum = ISO8601weeknum (&dec31ly);
+        }
+      break;
+    }
+
+  if (t->tm_mon == 11)
+    {
+      int wday, mday;
+
+      wday = t->tm_wday;
+      mday = t->tm_mday;
+      if ((wday == MONDAY && (mday >= 29 && mday <= 31))
+          || (wday == TUESDAY && (mday == 30 || mday == 31))
+          || (wday == WEDNESDAY && mday == 31))
+        wnum = 1;
+    }
+
+  return wnum;
+}
+
+/* Draw the weekly view inside calendar panel. */
+static void
+draw_weekly_view (struct window *cwin, struct date *current_day,
+                  unsigned sunday_first)
+{
+#define DAYSLICESNO  6  
+  const int WCALWIDTH = 30;
+  const int OFFY = 2 + (CALHEIGHT - 9) / 2;
+  struct tm t;
+  int OFFX, j, c_wday, days_to_remove, weeknum;
+
+  OFFX = (wins_sbar_width () - WCALWIDTH) / 2 + 1;
+  
+  /* Fill in a tm structure with the first day of the selected week. */
+  c_wday = calendar_get_wday (&slctd_day);
+  if (sunday_first)
+    days_to_remove = c_wday;
+  else
+    days_to_remove = c_wday == 0 ? WEEKINDAYS - 1 : c_wday - 1;
+  
+  (void)memset (&t, 0, sizeof (struct tm));
+  t.tm_mday = slctd_day.dd;
+  t.tm_mon = slctd_day.mm - 1;
+  t.tm_year = slctd_day.yyyy - 1900;
+  (void)mktime (&t);
+  (void)date_change (&t, 0, -days_to_remove);
+
+  /* Print the week number. */
+  weeknum = ISO8601weeknum (&t);
+  custom_apply_attr (cwin->p, ATTR_HIGHEST);      
+  mvwprintw (cwin->p, 2, cwin->w - 9, "(# %02d)", weeknum);
+  custom_remove_attr (cwin->p, ATTR_HIGHEST);
+  
+  /* Now draw calendar view. */
+  for (j = 0; j < WEEKINDAYS; j++)
+    {
+      struct date date;
+      unsigned attr, item_this_day;
+      int i, slices[DAYSLICESNO];
+      
+      /* print the day names, with regards to the first day of the week */
+      custom_apply_attr (cwin->p, ATTR_HIGHEST);      
+      mvwprintw (cwin->p, OFFY, OFFX + 4 * j, "%s",
+		 _(daynames[1 + j - sunday_first]));
+      custom_remove_attr (cwin->p, ATTR_HIGHEST);
+
+      /* Check if the day to be printed has an item or not. */
+      date.dd = t.tm_mday;
+      date.mm = t.tm_mon + 1;
+      date.yyyy = t.tm_year + 1900;
+      item_this_day = day_check_if_item (date);
+      
+      /* Print the day numbers with appropriate decoration. */
+      if (t.tm_mday == current_day->dd
+          && current_day->mm == slctd_day.mm
+          && current_day->yyyy == slctd_day.yyyy
+          && current_day->dd != slctd_day.dd)
+        attr = ATTR_LOWEST;               /* today, but not selected */
+      else if (t.tm_mday == slctd_day.dd)
+        attr = ATTR_HIGHEST;              /* selected day */
+      else if (item_this_day)
+        attr = ATTR_LOW;
+      else
+        attr = 0;
+
+      if (attr)
+        custom_apply_attr (cwin->p, attr);
+      mvwprintw (cwin->p, OFFY + 1, OFFX + 1 + 4 * j, "%02d", t.tm_mday);
+      if (attr)
+        custom_remove_attr (cwin->p, attr);
+
+      /* Draw slices indicating appointment times. */
+      bzero (slices, DAYSLICESNO * sizeof *slices);
+      if (day_chk_busy_slices (date, DAYSLICESNO, slices))
+        {
+          for (i = 0; i < DAYSLICESNO; i++)
+            {
+              if (j != WEEKINDAYS - 1 && i != DAYSLICESNO - 1)
+                mvwhline (cwin->p, OFFY + 2 + i, OFFX + 3 + 4 * j, ACS_S9, 2);
+              if (slices[i])
+                {
+                  int highlight;
+
+                  highlight = (t.tm_mday == slctd_day.dd) ? 1 : 0;
+                  if (highlight)
+                    custom_apply_attr (cwin->p, attr);
+                  wattron (cwin->p, A_REVERSE);
+                  mvwprintw (cwin->p, OFFY + 2 + i, OFFX + 1 + 4 * j, " ");
+                  mvwprintw (cwin->p, OFFY + 2 + i, OFFX + 2 + 4 * j, " ");
+                  wattroff (cwin->p, A_REVERSE);
+                  if (highlight)
+                    custom_remove_attr (cwin->p, attr);                    
+                }
+            }
+        }
+      
+      /* get next day */
+      (void)date_change (&t, 0, 1);
+    }
+
+  /* Draw marks to indicate midday on the sides of the calendar. */
+  custom_apply_attr (cwin->p, ATTR_HIGHEST);
+  mvwhline (cwin->p, OFFY + 1 + DAYSLICESNO / 2, OFFX, ACS_S9, 1);
+  mvwhline (cwin->p, OFFY + 1 + DAYSLICESNO / 2,
+            OFFX + WCALWIDTH - 3, ACS_S9, 1);
+  custom_remove_attr (cwin->p, ATTR_HIGHEST);
+
+#undef DAYSLICESNO  
+}
+
+/* Function used to display the calendar panel. */
+void
+calendar_update_panel (struct window *cwin)
+{
+  struct date current_day;
+  unsigned sunday_first;
+
+  calendar_store_current_date (&current_day);
+  erase_window_part (cwin->p, 1, 3, cwin->w - 2, cwin->h - 2);
+  mvwhline (cwin->p, 2, 1, ACS_HLINE, cwin->w - 2);
+  sunday_first = calendar_week_begins_on_monday () ? 0 : 1;
+
+  draw_calendar[calendar_view] (cwin, &current_day, sunday_first);
+
+  wnoutrefresh (cwin->p);
 }
 
 /* Set the selected day in calendar to current day. */
 void
 calendar_goto_today (void)
 {
-  date_t today;
+  struct date today;
 
   calendar_store_current_date (&today);
   slctd_day.dd = today.dd;
@@ -404,30 +675,8 @@ calendar_change_day (int datefmt)
   return;
 }
 
-/* 
- * Used to change date by adding a certain amount of days or weeks.
- * Returns 0 on success, 1 otherwise.
- */
-static int
-date_change (struct tm *date, int delta_month, int delta_day)
-{
-  struct tm t;
-
-  t = *date;
-  t.tm_mon += delta_month;
-  t.tm_mday += delta_day;
-
-  if (mktime (&t) == -1)
-    return (1);
-  else
-    {
-      *date = t;
-      return (0);
-    }
-}
-
 void
-calendar_move (move_t move)
+calendar_move (enum move move)
 {
   int ret, days_to_remove, days_to_add;
   struct tm t;
@@ -648,7 +897,7 @@ pom (time_t tmpt)
   struct tm *GMT;
   double days;
   int cnt;
-  pom_e pom;
+  enum pom pom;
 
   pom = NO_POM;
   GMT = gmtime (&tmpt);
@@ -672,7 +921,7 @@ char *
 calendar_get_pom (time_t date)
 {
   char *pom_pict[MOON_PHASES] = { "   ", "|) ", "(|)", "(| ", " | " };
-  pom_e phase = NO_POM;
+  enum pom phase = NO_POM;
   double pom_today, relative_pom, pom_yesterday, pom_tomorrow;
   const double half = 50.0;
 
