@@ -1,9 +1,9 @@
-/*	$calcurse: custom.c,v 1.44 2009-08-02 09:29:24 culot Exp $	*/
+/*	$calcurse: custom.c,v 1.51 2010-03-29 07:37:16 culot Exp $	*/
 
 /*
  * Calcurse - text-based organizer
  *
- * Copyright (c) 2004-2009 Frederic Culot <frederic@culot.org>
+ * Copyright (c) 2004-2010 Frederic Culot <frederic@culot.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -40,20 +40,40 @@
 #include <stdlib.h>
 #include <math.h>
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif /* HAVE_CONFIG_H */
+#include "calcurse.h"
 
-#include "i18n.h"
-#include "io.h"
-#include "utils.h"
-#include "keys.h"
-#include "apoint.h"
-#include "help.h"
-#include "mem.h"
-#include "custom.h"
+/* Available configuration variables. */
+enum conf_var {
+  CUSTOM_CONF_NOVARIABLE,
+  CUSTOM_CONF_AUTOSAVE,
+  CUSTOM_CONF_PERIODICSAVE,
+  CUSTOM_CONF_CONFIRMQUIT,
+  CUSTOM_CONF_CONFIRMDELETE,
+  CUSTOM_CONF_SKIPSYSTEMDIALOGS,
+  CUSTOM_CONF_SKIPPROGRESSBAR,
+  CUSTOM_CONF_CALENDAR_DEFAULTVIEW,
+  CUSTOM_CONF_WEEKBEGINSONMONDAY,
+  CUSTOM_CONF_COLORTHEME,
+  CUSTOM_CONF_LAYOUT,
+  CUSTOM_CONF_SBAR_WIDTH,
+  CUSTOM_CONF_NOTIFYBARSHOW,
+  CUSTOM_CONF_NOTIFYBARDATE,
+  CUSTOM_CONF_NOTIFYBARCLOCK,
+  CUSTOM_CONF_NOTIFYBARWARNING,
+  CUSTOM_CONF_NOTIFYBARCOMMAND,
+  CUSTOM_CONF_OUTPUTDATEFMT,
+  CUSTOM_CONF_INPUTDATEFMT,
+  CUSTOM_CONF_DMON_ENABLE,
+  CUSTOM_CONF_DMON_LOG,
+  CUSTOM_CONF_VARIABLES
+};
 
-static struct attribute_s attr;
+struct attribute {
+  int color[7];
+  int nocolor[7];
+};
+
+static struct attribute attr;
 
 static unsigned
 fill_config_var (char *string)
@@ -217,20 +237,20 @@ custom_remove_attr (WINDOW *win, int attr_num)
 
 /* Load the user configuration. */
 void
-custom_load_conf (conf_t *conf, int background)
+custom_load_conf (struct conf *conf, int background)
 {
   FILE *data_file;
   char *mesg_line1 = _("Failed to open config file");
   char *mesg_line2 = _("Press [ENTER] to continue");
   char buf[BUFSIZ], e_conf[BUFSIZ];
-  int var;
+  enum conf_var var;
 
   data_file = fopen (path_conf, "r");
   if (data_file == NULL)
     {
       status_mesg (mesg_line1, mesg_line2);
       wnoutrefresh (win[STA].p);
-      doupdate ();
+      wins_doupdate ();
       (void)keys_getch (win[STA].p);
     }
   var = CUSTOM_CONF_NOVARIABLE;
@@ -273,6 +293,10 @@ custom_load_conf (conf_t *conf, int background)
 	  conf->skip_progress_bar = fill_config_var (e_conf);
 	  var = 0;
 	  break;
+        case CUSTOM_CONF_CALENDAR_DEFAULTVIEW:
+          calendar_set_view (atoi (e_conf));
+          var = 0;
+          break;
 	case CUSTOM_CONF_WEEKBEGINSONMONDAY:
 	  if (fill_config_var (e_conf))
 	    calendar_set_first_day_of_week (MONDAY);
@@ -288,6 +312,10 @@ custom_load_conf (conf_t *conf, int background)
 	  wins_set_layout (atoi (e_conf));
 	  var = 0;
 	  break;
+        case CUSTOM_CONF_SBAR_WIDTH:
+          wins_set_sbar_width (atoi (e_conf));
+          var = 0;
+          break;
 	case CUSTOM_CONF_NOTIFYBARSHOW:
 	  nbar.show = fill_config_var (e_conf);
 	  var = 0;
@@ -344,12 +372,16 @@ custom_load_conf (conf_t *conf, int background)
 	var = CUSTOM_CONF_SKIPSYSTEMDIALOGS;
       else if (strncmp (e_conf, "skip_progress_bar=", 18) == 0)
 	var = CUSTOM_CONF_SKIPPROGRESSBAR;
-      else if (strncmp (e_conf, "week_begins_on_monday=", 23) == 0)
+      else if (strncmp (e_conf, "calendar_default_view=", 22) == 0)
+        var = CUSTOM_CONF_CALENDAR_DEFAULTVIEW;
+      else if (strncmp (e_conf, "week_begins_on_monday=", 22) == 0)
 	var = CUSTOM_CONF_WEEKBEGINSONMONDAY;
       else if (strncmp (e_conf, "color-theme=", 12) == 0)
 	var = CUSTOM_CONF_COLORTHEME;
       else if (strncmp (e_conf, "layout=", 7) == 0)
 	var = CUSTOM_CONF_LAYOUT;
+      else if (strncmp (e_conf, "side-bar_width=", 15) == 0)
+        var = CUSTOM_CONF_SBAR_WIDTH;
       else if (strncmp (e_conf, "notify-bar_show=", 16) == 0)
 	var = CUSTOM_CONF_NOTIFYBARSHOW;
       else if (strncmp (e_conf, "notify-bar_date=", 16) == 0)
@@ -377,44 +409,44 @@ custom_load_conf (conf_t *conf, int background)
 void
 custom_config_bar (void)
 {
-  int smlspc, spc;
-
-  smlspc = 2;
-  spc = 15;
+  const int SMLSPC = 2;
+  const int SPC = 15;
 
   custom_apply_attr (win[STA].p, ATTR_HIGHEST);
   mvwprintw (win[STA].p, 0, 2, "Q");
   mvwprintw (win[STA].p, 1, 2, "G");
-  mvwprintw (win[STA].p, 0, 2 + spc, "L");
-  mvwprintw (win[STA].p, 1, 2 + spc, "C");
-  mvwprintw (win[STA].p, 0, 2 + 2 * spc, "N");
-  mvwprintw (win[STA].p, 1, 2 + 2 * spc, "K");  
+  mvwprintw (win[STA].p, 0, 2 + SPC, "L");
+  mvwprintw (win[STA].p, 1, 2 + SPC, "S");
+  mvwprintw (win[STA].p, 0, 2 + 2 * SPC, "C");
+  mvwprintw (win[STA].p, 1, 2 + 2 * SPC, "N");
+  mvwprintw (win[STA].p, 0, 2 + 3 * SPC, "K");  
   custom_remove_attr (win[STA].p, ATTR_HIGHEST);
 
-  mvwprintw (win[STA].p, 0, 2 + smlspc, _("Exit"));
-  mvwprintw (win[STA].p, 1, 2 + smlspc, _("General"));
-  mvwprintw (win[STA].p, 0, 2 + spc + smlspc, _("Layout"));
-  mvwprintw (win[STA].p, 1, 2 + spc + smlspc, _("Color"));
-  mvwprintw (win[STA].p, 0, 2 + 2 * spc + smlspc, _("Notify"));
-  mvwprintw (win[STA].p, 1, 2 + 2 * spc + smlspc, _("Keys"));
+  mvwprintw (win[STA].p, 0, 2 + SMLSPC, _("Exit"));
+  mvwprintw (win[STA].p, 1, 2 + SMLSPC, _("General"));
+  mvwprintw (win[STA].p, 0, 2 + SPC + SMLSPC, _("Layout"));
+  mvwprintw (win[STA].p, 1, 2 + SPC + SMLSPC, _("Sidebar"));
+  mvwprintw (win[STA].p, 0, 2 + 2 * SPC + SMLSPC, _("Color"));  
+  mvwprintw (win[STA].p, 1, 2 + 2 * SPC + SMLSPC, _("Notify"));
+  mvwprintw (win[STA].p, 0, 2 + 3 * SPC + SMLSPC, _("Keys"));
   
   wnoutrefresh (win[STA].p);
   wmove (win[STA].p, 0, 0);
-  doupdate ();
+  wins_doupdate ();
 }
 
 static void
 layout_selection_bar (void)
 {
-  binding_t quit    = {_("Exit"),     KEY_GENERIC_QUIT};
-  binding_t select  = {_("Select"),   KEY_GENERIC_SELECT};
-  binding_t up      = {_("Up"),       KEY_MOVE_UP};
-  binding_t down    = {_("Down"),     KEY_MOVE_DOWN};
-  binding_t left    = {_("Left"),     KEY_MOVE_LEFT};
-  binding_t right   = {_("Right"),    KEY_MOVE_RIGHT};
-  binding_t help    = {_("Help"),     KEY_GENERIC_HELP};
+  struct binding quit    = {_("Exit"),     KEY_GENERIC_QUIT};
+  struct binding select  = {_("Select"),   KEY_GENERIC_SELECT};
+  struct binding up      = {_("Up"),       KEY_MOVE_UP};
+  struct binding down    = {_("Down"),     KEY_MOVE_DOWN};
+  struct binding left    = {_("Left"),     KEY_MOVE_LEFT};
+  struct binding right   = {_("Right"),    KEY_MOVE_RIGHT};
+  struct binding help    = {_("Help"),     KEY_GENERIC_HELP};
   
-  binding_t *binding[] = {&quit, &select, &up, &down, &left, &right, &help};
+  struct binding *binding[] = {&quit, &select, &up, &down, &left, &right, &help};
   int binding_size = sizeof (binding) / sizeof (binding[0]);
 
   keys_display_bindings_bar (win[STA].p, binding, 0, binding_size);
@@ -425,7 +457,8 @@ layout_selection_bar (void)
 
 /* Used to display available layouts in layout configuration menu. */
 static void
-display_layout_config (window_t *lwin, int mark, int cursor, int need_reset)
+display_layout_config (struct window *lwin, int mark, int cursor,
+                       int need_reset)
 {
 #define CURSOR			(32 | A_REVERSE)
 #define MARK			88
@@ -485,7 +518,7 @@ display_layout_config (window_t *lwin, int mark, int cursor, int need_reset)
   layout_selection_bar ();
   wnoutrefresh (win[STA].p);
   wnoutrefresh (lwin->p);
-  doupdate ();
+  wins_doupdate ();
   if (notify_bar ())
     notify_update_bar ();
 }
@@ -494,8 +527,8 @@ display_layout_config (window_t *lwin, int mark, int cursor, int need_reset)
 void
 custom_layout_config (void)
 {
-  scrollwin_t hwin;
-  window_t conf_win;
+  struct scrollwin hwin;
+  struct window conf_win;
   int ch, mark, cursor, need_reset;
   char *help_text =
     _("With this configuration menu, one can choose where panels will be\n"
@@ -519,7 +552,7 @@ custom_layout_config (void)
 	{
 	case KEY_RESIZE:
 	  endwin ();
-	  refresh ();
+	  wins_refresh ();
 	  curs_set (0);
 	  need_reset = 1;
 	  break;
@@ -565,8 +598,62 @@ custom_layout_config (void)
 #undef NBLAYOUTS
 #undef LAYOUTSPERCOL
 
+/* Sidebar configuration screen. */
+void
+custom_sidebar_config (void)
+{
+  struct scrollwin hwin;
+  struct binding quit = {_("Exit"),     KEY_GENERIC_QUIT};
+  struct binding inc  = {_("Width +"),    KEY_MOVE_UP};  
+  struct binding dec  = {_("Width -"),    KEY_MOVE_DOWN};
+  struct binding help = {_("Help"),       KEY_GENERIC_HELP};
+  struct binding *binding[] = {&inc, &dec, &help, &quit};
+  char *help_text =
+    _("This configuration screen is used to change the width of the side bar.\n"
+      "The side bar is the part of the screen which contains two panels:\n"
+      "the calendar and, depending on the chosen layout, either the todo list\n"
+      "or the appointment list.\n\n"
+      "The side bar width can be up to 50% of the total screen width, but\n"
+      "can't be smaller than " TOSTRING(SBARMINWIDTH) " characters wide.\n\n");
+  int ch, binding_size;
+
+  binding_size = sizeof (binding) / sizeof (binding[0]);
+
+  keys_display_bindings_bar (win[STA].p, binding, 0, binding_size);
+  wins_doupdate ();
+
+  while ((ch = keys_getch (win[STA].p)) != KEY_GENERIC_QUIT)  
+    {
+      switch (ch)
+        {
+        case KEY_MOVE_UP:
+          wins_sbar_winc ();
+          break;
+        case KEY_MOVE_DOWN:
+          wins_sbar_wdec ();
+          break;
+        case KEY_GENERIC_HELP:
+          help_wins_init (&hwin, 0, 0,
+                          (notify_bar ()) ? row - 3 : row - 2, col);
+          mvwprintw (hwin.pad.p, 1, 0, "%s", help_text);
+          hwin.total_lines = 6;
+          wins_scrollwin_display (&hwin);
+          wgetch (hwin.win.p);
+          wins_scrollwin_delete (&hwin);
+          break;
+        default:
+          continue;
+        }
+      wins_reinit_panels ();
+      wins_update_border ();
+      wins_update_panels ();
+      keys_display_bindings_bar (win[STA].p, binding, 0, binding_size);
+      wins_doupdate ();
+    }
+}
+
 static void
-set_confwin_attr (window_t *cwin)
+set_confwin_attr (struct window *cwin)
 {
   cwin->h = (notify_bar ())? row - 3 : row - 2;
   cwin->w = col;
@@ -578,7 +665,7 @@ set_confwin_attr (window_t *cwin)
  * (useful in case of window resize).
  */
 void
-custom_confwin_init (window_t *confwin, char *label)
+custom_confwin_init (struct window *confwin, char *label)
 {
   if (confwin->p)
     {
@@ -605,16 +692,16 @@ custom_confwin_init (window_t *confwin, char *label)
 static void
 color_selection_bar (void)
 {
-  binding_t quit    = {_("Exit"),     KEY_GENERIC_QUIT};
-  binding_t select  = {_("Select"),   KEY_GENERIC_SELECT};
-  binding_t nocolor = {_("No color"), KEY_GENERIC_CANCEL};
-  binding_t up      = {_("Up"),       KEY_MOVE_UP};
-  binding_t down    = {_("Down"),     KEY_MOVE_DOWN};
-  binding_t left    = {_("Left"),     KEY_MOVE_LEFT};
-  binding_t right   = {_("Right"),    KEY_MOVE_RIGHT};
+  struct binding quit    = {_("Exit"),     KEY_GENERIC_QUIT};
+  struct binding select  = {_("Select"),   KEY_GENERIC_SELECT};
+  struct binding nocolor = {_("No color"), KEY_GENERIC_CANCEL};
+  struct binding up      = {_("Up"),       KEY_MOVE_UP};
+  struct binding down    = {_("Down"),     KEY_MOVE_DOWN};
+  struct binding left    = {_("Left"),     KEY_MOVE_LEFT};
+  struct binding right   = {_("Right"),    KEY_MOVE_RIGHT};
 
   
-  binding_t *binding[] = {
+  struct binding *binding[] = {
     &quit, &nocolor, &up, &down, &left, &right, &select
   };
   int binding_size = sizeof (binding) / sizeof (binding[0]);
@@ -627,7 +714,7 @@ color_selection_bar (void)
  * This is useful for window resizing.
  */
 static void
-display_color_config (window_t *cwin, int *mark_fore, int *mark_back,
+display_color_config (struct window *cwin, int *mark_fore, int *mark_back,
 		      int cursor, int need_reset, int theme_changed)
 {
 #define	SIZE 			(2 * (NBUSERCOLORS + 1))
@@ -746,7 +833,7 @@ display_color_config (window_t *cwin, int *mark_fore, int *mark_back,
   color_selection_bar ();
   wnoutrefresh (win[STA].p);
   wnoutrefresh (cwin->p);
-  doupdate ();
+  wins_doupdate ();
   if (notify_bar ())
     notify_update_bar ();
 }
@@ -755,7 +842,7 @@ display_color_config (window_t *cwin, int *mark_fore, int *mark_back,
 void
 custom_color_config (void)
 {
-  window_t conf_win;
+  struct window conf_win;
   int ch, cursor, need_reset, theme_changed;
   int mark_fore, mark_back;
 
@@ -779,7 +866,7 @@ custom_color_config (void)
 	{
 	case KEY_RESIZE:
 	  endwin ();
-	  refresh ();
+	  wins_refresh ();
 	  curs_set (0);
 	  need_reset = 1;
 	  break;
@@ -880,7 +967,7 @@ custom_color_theme_name (char *theme_name)
 
 /* Prints the general options. */
 static int
-print_general_options (WINDOW *win, conf_t *conf)
+print_general_options (WINDOW *win, struct conf *conf)
 {
   enum {
     AUTO_SAVE,
@@ -980,7 +1067,7 @@ print_general_options (WINDOW *win, conf_t *conf)
 }
 
 void
-custom_set_swsiz (scrollwin_t *sw)
+custom_set_swsiz (struct scrollwin *sw)
 {
   sw->win.x = 0;
   sw->win.y = 0;
@@ -995,9 +1082,9 @@ custom_set_swsiz (scrollwin_t *sw)
                           
 /* General configuration. */
 void
-custom_general_config (conf_t *conf)
+custom_general_config (struct conf *conf)
 {
-  scrollwin_t cwin;
+  struct scrollwin cwin;
   char *number_str =
     _("Enter an option number to change its value");
   char *keys =
@@ -1182,16 +1269,16 @@ print_keys_bindings (WINDOW *win, int selected_row, int selected_elm, int yoff)
 static void
 custom_keys_config_bar (void)
 {
-  binding_t quit  = {_("Exit"),     KEY_GENERIC_QUIT};
-  binding_t info  = {_("Key info"), KEY_GENERIC_HELP};
-  binding_t add   = {_("Add key"),  KEY_ADD_ITEM};
-  binding_t del   = {_("Del key"),  KEY_DEL_ITEM};
-  binding_t up    = {_("Up"),       KEY_MOVE_UP};
-  binding_t down  = {_("Down"),     KEY_MOVE_DOWN};
-  binding_t left  = {_("Prev Key"), KEY_MOVE_LEFT};
-  binding_t right = {_("Next Key"), KEY_MOVE_RIGHT};
+  struct binding quit  = {_("Exit"),     KEY_GENERIC_QUIT};
+  struct binding info  = {_("Key info"), KEY_GENERIC_HELP};
+  struct binding add   = {_("Add key"),  KEY_ADD_ITEM};
+  struct binding del   = {_("Del key"),  KEY_DEL_ITEM};
+  struct binding up    = {_("Up"),       KEY_MOVE_UP};
+  struct binding down  = {_("Down"),     KEY_MOVE_DOWN};
+  struct binding left  = {_("Prev Key"), KEY_MOVE_LEFT};
+  struct binding right = {_("Next Key"), KEY_MOVE_RIGHT};
     
-  binding_t *binding[] = {
+  struct binding *binding[] = {
     &quit, &info, &add, &del, &up, &down, &left, &right
   };
   int binding_size = sizeof (binding) / sizeof (binding[0]);
@@ -1202,7 +1289,7 @@ custom_keys_config_bar (void)
 void
 custom_keys_config (void)
 {
-  scrollwin_t kwin;
+  struct scrollwin kwin;
   int selrow, selelm, firstrow, lastrow, nbrowelm, nbdisplayed;
   int keyval, used, not_recognized;
   char *keystr;
@@ -1297,7 +1384,7 @@ custom_keys_config (void)
               used = keys_assign_binding (keyval, selrow);
               if (used)
                 {
-                  keys_e action;
+                  enum key action;
                   
                   action = keys_get_action (keyval);
                   WARN_MSG (_("This key is already in use for %s, "
