@@ -1,9 +1,9 @@
-/*	$calcurse: help.c,v 1.41 2009-07-12 17:55:14 culot Exp $	*/
+/*	$calcurse: help.c,v 1.44 2010-03-21 10:17:03 culot Exp $	*/
 
 /*
  * Calcurse - text-based organizer
  *
- * Copyright (c) 2004-2009 Frederic Culot <frederic@culot.org>
+ * Copyright (c) 2004-2010 Frederic Culot <frederic@culot.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -41,16 +41,14 @@
 #include <ctype.h>
 #include <sys/types.h>
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif /* HAVE_CONFIG_H */
+#include "calcurse.h"
 
-#include "i18n.h"
-#include "custom.h"
-#include "utils.h"
-#include "keys.h"
-#include "notify.h"
-#include "help.h"
+#define HELPTEXTSIZ 4096
+
+typedef struct {
+  char *title;
+  char text[HELPTEXTSIZ];
+} help_page_t;
 
 typedef enum
 {
@@ -100,7 +98,7 @@ get_help_lines (char *text)
  * of lines that were written. 
  */
 static int
-help_write_pad (window_t *win, char *title, char *text, keys_e action)
+help_write_pad (struct window *win, char *title, char *text, enum key action)
 {
   int colnum, rownum;
   char *bindings_title = "key bindings: %s";
@@ -150,7 +148,7 @@ help_write_pad (window_t *win, char *title, char *text, keys_e action)
  * faster. 
  */
 void
-help_wins_init (scrollwin_t *hwin, int x, int y, int h, int w)
+help_wins_init (struct scrollwin *hwin, int x, int y, int h, int w)
 {
   const int PADOFFSET = 4;
   const int TITLELINES = 3;
@@ -175,7 +173,7 @@ help_wins_init (scrollwin_t *hwin, int x, int y, int h, int w)
  * size and placement.
  */
 static void
-help_wins_reinit (scrollwin_t *hwin)
+help_wins_reinit (struct scrollwin *hwin)
 {
   wins_scrollwin_delete (hwin);
   wins_get_config ();
@@ -184,10 +182,10 @@ help_wins_reinit (scrollwin_t *hwin)
 
 /* Reset the screen, needed when resizing terminal for example. */
 static void
-help_wins_reset (scrollwin_t *hwin)
+help_wins_reset (struct scrollwin *hwin)
 {
   endwin ();
-  refresh ();
+  wins_refresh ();
   curs_set (0);
   delwin (win[STA].p);
   help_wins_reinit (hwin);
@@ -326,9 +324,9 @@ help_screen (void)
     MOVE_RIGHT,
     DIRECTIONS
   };
-  scrollwin_t hwin;
+  struct scrollwin hwin;
   int need_resize;
-  keys_e ch = KEY_GENERIC_HELP;
+  enum key ch = KEY_GENERIC_HELP;
   int page, oldpage;
   help_page_t hscr[HELPSCREENS];
   char keystr[DIRECTIONS][BUFSIZ];
@@ -688,7 +686,11 @@ help_screen (void)
       " '%s' : +1 Day          -> move to next day\n"
       " '%s' : -1 Week         -> move to previous week\n"
       " '%s' : +1 Week         -> move to next week\n"
-      " '%s' : Goto today      -> move to current day"),
+      " '%s' : Goto today      -> move to current day\n"
+      "\nThe '%s' and '%s' keys are used to scroll text upward or downward\n"
+      "when inside specific screens such the help screens for example.\n"
+      "They are also used when the calendar screen is selected to switch\n"
+      "between the available views (monthly and weekly calendar views)."),
             keys_action_firstkey (KEY_GENERIC_REDRAW),
             keys_action_firstkey (KEY_GENERIC_ADD_APPT),
             keys_action_firstkey (KEY_GENERIC_ADD_TODO),
@@ -696,7 +698,9 @@ help_screen (void)
             keys_action_firstkey (KEY_GENERIC_NEXT_DAY),
             keys_action_firstkey (KEY_GENERIC_PREV_WEEK),
             keys_action_firstkey (KEY_GENERIC_NEXT_WEEK),
-            keys_action_firstkey (KEY_GENERIC_GOTO_TODAY));
+            keys_action_firstkey (KEY_GENERIC_GOTO_TODAY),
+            keys_action_firstkey (KEY_GENERIC_SCROLL_UP),
+            keys_action_firstkey (KEY_GENERIC_SCROLL_DOWN));
 
   hscr[HELP_OTHER].title = _("OtherCmd\n");
   (void)snprintf (hscr[HELP_OTHER].text, HELPTEXTSIZ,
