@@ -1,9 +1,9 @@
-/*	$calcurse: utils.c,v 1.82 2009-08-01 17:53:11 culot Exp $	*/
+/*	$calcurse: utils.c,v 1.84 2010-03-21 10:17:04 culot Exp $	*/
 
 /*
  * Calcurse - text-based organizer
  *
- * Copyright (c) 2004-2009 Frederic Culot <frederic@culot.org>
+ * Copyright (c) 2004-2010 Frederic Culot <frederic@culot.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -45,20 +45,7 @@
 #include <sys/types.h>
 #include <errno.h>
 
-#include "utils.h"
-#include "i18n.h"
-#include "wins.h"
-#include "custom.h"
-#include "keys.h"
-#include "io.h"
-#include "recur.h"
-#include "event.h"
-#include "apoint.h"
-#include "todo.h"
-#include "day.h"
-#include "keys.h"
-#include "dmon.h"
-#include "mem.h"
+#include "calcurse.h"
 
 /* General routine to exit calcurse properly. */
 void
@@ -70,7 +57,7 @@ exit_calcurse (int status)
     {
       notify_stop_main_thread ();
       clear ();
-      refresh ();
+      wins_refresh ();
       endwin ();
       ui_mode = UI_CMDLINE;
       was_interactive = 1;
@@ -134,10 +121,10 @@ fatalbox (const char *errmsg)
   mvwprintw (errwin, 3, 1, reportmsg);
   mvwprintw (errwin, 5, (WINCOL - strlen (msg)) / 2, "%s", msg);
   custom_remove_attr (errwin, ATTR_HIGHEST);
-  wrefresh (errwin);
+  wins_wrefresh (errwin);
   (void)wgetch (errwin);
   delwin (errwin);
-  doupdate ();
+  wins_doupdate ();
 }
 
 void
@@ -160,10 +147,10 @@ warnbox (const char *msg)
   wins_show (warnwin, label);
   mvwprintw (warnwin, 5, (WINCOL - strlen (displmsg)) / 2, "%s", displmsg);
   custom_remove_attr (warnwin, ATTR_HIGHEST);
-  wrefresh (warnwin);
+  wins_wrefresh (warnwin);
   (void)wgetch (warnwin);
   delwin (warnwin);
-  doupdate ();
+  wins_doupdate ();
 }
 
 /* 
@@ -219,8 +206,7 @@ popup (int pop_row, int pop_col, int pop_y, int pop_x, char *title, char *msg,
     mvwprintw (popup_win, pop_row - 2, pop_col - (strlen (any_key) + 1), "%s",
                any_key);
   custom_remove_attr (popup_win, ATTR_HIGHEST);
-  wrefresh (popup_win);
-  doupdate ();
+  wins_wrefresh (popup_win);
   
   return popup_win;
 }
@@ -331,7 +317,7 @@ add_char (int pos, int ch, char *str)
  * environment, otherwise the cursor would move from place to place without
  * control.
  */
-int
+enum getstr
 getstring (WINDOW *win, char *str, int l, int x, int y)
 {
   int ch, newpos, len = 0;
@@ -422,7 +408,7 @@ getstring (WINDOW *win, char *str, int l, int x, int y)
 
 	}
       showstring (win, y, x, orig, len, newpos);
-      doupdate ();
+      wins_doupdate ();
     }
   *str = 0;
   custom_remove_attr (win, ATTR_HIGHEST);
@@ -503,7 +489,7 @@ get_item_min (long date)
 }
 
 long
-date2sec (date_t day, unsigned hour, unsigned min)
+date2sec (struct date day, unsigned hour, unsigned min)
 {
   struct tm start, *lt;
   time_t tstart, t;
@@ -622,7 +608,7 @@ update_time_in_date (long date, unsigned hr, unsigned mn)
  * If no date is entered, current date is chosen.
  */
 long
-get_sec_date (date_t date)
+get_sec_date (struct date date)
 {
   struct tm *ptrtime;
   time_t timer;
@@ -739,7 +725,7 @@ item_in_popup (char *saved_a_start, char *saved_a_end, char *msg,
   mvwprintw (pad, 0, margin_left, "%s", msg);
   wmove (win[STA].p, 0, 0);
   pnoutrefresh (pad, 0, 0, margin_top + 2, margin_left, padl, winw);
-  doupdate ();
+  wins_doupdate ();
   (void)wgetch (popup_win);
   delwin (pad);
   delwin (popup_win);
@@ -752,7 +738,7 @@ get_today (void)
   struct tm *lt;
   time_t current_time;
   long current_day;
-  date_t day;
+  struct date day;
 
   current_time = time (NULL);
   lt = localtime (&current_time);
@@ -826,7 +812,7 @@ print_bool_option_incolor (WINDOW *win, unsigned option, int pos_y, int pos_x)
   mvwprintw (win, pos_y, pos_x, "%s", option_value);
   custom_remove_attr (win, color);
   wnoutrefresh (win);
-  doupdate ();
+  wins_doupdate ();
 }
 
 /* 
@@ -866,7 +852,7 @@ new_tempfile (const char *prefix, int trailing_len)
 
 /* Erase a note previously attached to a todo, event or appointment. */
 void
-erase_note (char **note, erase_flag_e flag)
+erase_note (char **note, enum eraseflg flag)
 {
   char fullname[BUFSIZ];
 
@@ -888,7 +874,8 @@ erase_note (char **note, erase_flag_e flag)
  * Returns 1 if sucessfully converted or 0 if the string is an invalid date.
  */
 int
-parse_date (char *date_string, int datefmt, int *year, int *month, int *day)
+parse_date (char *date_string, enum datefmt datefmt, int *year, int *month,
+            int *day)
 {
   int in1, in2, in3;
   int lyear, lmonth, lday;
