@@ -1,9 +1,9 @@
-/*	$calcurse: wins.c,v 1.26 2009-07-12 16:22:03 culot Exp $	*/
+/*	$calcurse: wins.c,v 1.33 2010-03-23 13:08:46 culot Exp $	*/
 
 /*
  * Calcurse - text-based organizer
  *
- * Copyright (c) 2007-2009 Frederic Culot <frederic@culot.org>
+ * Copyright (c) 2007-2010 Frederic Culot <frederic@culot.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -40,26 +40,90 @@
 #include <string.h>
 #include <math.h>
 
-#include "i18n.h"
-#include "keys.h"
-#include "notify.h"
-#include "utils.h"
-#include "todo.h"
-#include "custom.h"
-#include "mem.h"
-#include "wins.h"
+#include "calcurse.h"
 
 /* Variables to handle calcurse windows. */
-window_t win[NBWINS];
+struct window win[NBWINS];
 
-static window_e slctd_win;
+/* User-configurable side bar width. */
+static unsigned sbarwidth;
+
+static enum win slctd_win;
 static int layout;
 
+/*
+ * The screen_mutex mutex and wins_refresh(), wins_wrefresh(), wins_doupdate()
+ * functions are used to prevent concurrent updates of the screen.
+ * It was observed that the display could get screwed up when mulitple threads
+ * tried to refresh the screen at the same time.
+ *
+ * Note (2010-03-21):
+ * Since recent versions of ncurses (5.7), rudimentary support for threads are
+ * available (use_screen(), use_window() for instance - see curs_threads(3)),
+ * but to remain compatible with earlier versions, it was chosen to rely on a
+ * screen-level mutex instead.
+ */
+static pthread_mutex_t screen_mutex = PTHREAD_MUTEX_INITIALIZER;
+
+static unsigned
+screen_acquire (void)
+{
+  if (pthread_mutex_lock (&screen_mutex) != 0)
+    return 0;
+  else
+    return 1;
+}
+
+static void
+screen_release (void)
+{
+  (void)pthread_mutex_unlock (&screen_mutex);
+}
+
+int
+wins_refresh (void)
+{
+  int rc;
+  
+  if (!screen_acquire ())
+    return ERR;
+  rc = refresh ();
+  screen_release ();
+
+  return rc;
+}
+
+int
+wins_wrefresh (WINDOW *win)
+{
+  int rc;
+  
+  if (!win || !screen_acquire ())
+    return ERR;
+  rc = wrefresh (win);
+  screen_release ();
+
+  return rc;
+}
+
+int
+wins_doupdate (void)
+{
+  int rc;
+
+  if (!screen_acquire ())
+    return ERR;
+  rc = doupdate ();
+  screen_release ();
+
+  return rc;
+}
+
 /* Get the current layout. */
 int
 wins_layout (void)
 {
-  return (layout);
+  return layout;
 }
 
 /* Set the current layout. */
@@ -69,6 +133,65 @@ wins_set_layout (int nb)
   layout = nb;
 }
 
+/* Get the current side bar width. */
+unsigned
+wins_sbar_width (void)
+{
+  return sbarwidth ? sbarwidth : SBARMINWIDTH;
+}
+
+/*
+ * Return the side bar width in percentage of the total number of columns
+ * available in calcurse's screen.
+ */
+unsigned
+wins_sbar_wperc (void)
+{
+  unsigned perc;
+
+  perc = col ? (unsigned)(100 * sbarwidth / col + 1): 0;
+  
+  return perc > SBARMAXWIDTHPERC ? SBARMAXWIDTHPERC : perc;
+}
+
+/*
+ * Set side bar width (unit: number of characters) given a width in percentage
+ * of calcurse's screen total width.
+ * The side bar could not have a width representing more than 50% of the screen,
+ * and could not be less than SBARMINWIDTH characters.
+ */
+void
+wins_set_sbar_width (unsigned perc)
+{
+  if (perc > SBARMAXWIDTHPERC)
+    sbarwidth = col * SBARMAXWIDTHPERC / 100;
+  else if (perc <= 0)
+    sbarwidth = SBARMINWIDTH;
+  else
+    {
+      sbarwidth = (unsigned)(col * perc / 100);
+      if (sbarwidth < SBARMINWIDTH)
+        sbarwidth = SBARMINWIDTH;
+    }
+}
+
+/* Change the width of the side bar within acceptable boundaries. */
+void
+wins_sbar_winc (void)
+{
+  if (sbarwidth < SBARMINWIDTH)
+    sbarwidth = SBARMINWIDTH + 1;
+  else if (sbarwidth < SBARMAXWIDTHPERC * col / 100)
+    sbarwidth++;
+}
+
+void
+wins_sbar_wdec (void)
+{
+  if (sbarwidth > SBARMINWIDTH)
+    sbarwidth--;
+}
+
 /* Initialize the selected window in calcurse's interface. */
 void
 wins_slctd_init (void)
@@ -77,7 +200,7 @@ wins_slctd_init (void)
 }
 
 /* Returns an enum which corresponds to the window which is selected. */
-window_e
+enum win
 wins_slctd (void)
 {
   return (slctd_win);
@@ -85,7 +208,7 @@ wins_slctd (void)
 
 /* Sets the selected window. */
 void
-wins_slctd_set (window_e window)
+wins_slctd_set (enum win window)
 {
   slctd_win = window;
 }
@@ -100,17 +223,12 @@ wins_slctd_next (void)
     slctd_win++;
 }
 
-/* Create all the windows. */
-void
-wins_init (void)
+static void
+wins_init_panels (void)
 {
   char label[BUFSIZ];
-
-  /* 
-   * Create the three main windows plus the status bar and the pad used to
-   * display appointments and event. 
-   */
-  win[CAL].p = newwin (CALHEIGHT, CALWIDTH, win[CAL].y, win[CAL].x);
+  
+  win[CAL].p = newwin (CALHEIGHT, wins_sbar_width (), win[CAL].y, win[CAL].x);
   (void)snprintf (label, BUFSIZ, _("Calendar"));
   wins_show (win[CAL].p, label);
 
@@ -124,12 +242,19 @@ wins_init (void)
   (void)snprintf (label, BUFSIZ, _("ToDo"));
   wins_show (win[TOD].p, label);
 
-  win[STA].p = newwin (win[STA].h, win[STA].w, win[STA].y, win[STA].x);
-
-  /* Enable function keys (i.e. arrow keys) in those windows */
+  /* Enable function keys (i.e. arrow keys) in those windows */  
   keypad (win[CAL].p, TRUE);
   keypad (win[APP].p, TRUE);
   keypad (win[TOD].p, TRUE);
+}
+
+/* Create all the windows. */
+void
+wins_init (void)
+{
+  wins_init_panels ();
+  win[STA].p = newwin (win[STA].h, win[STA].w, win[STA].y, win[STA].x);
+
   keypad (win[STA].p, TRUE);
 
   /* Notify that the curses mode is now launched. */
@@ -141,7 +266,7 @@ wins_init (void)
  * scrolling faster.
  */
 void
-wins_scrollwin_init (scrollwin_t *sw)
+wins_scrollwin_init (struct scrollwin *sw)
 {
   EXIT_IF (sw == 0, "null pointer");
   sw->win.p = newwin (sw->win.h, sw->win.w, sw->win.y, sw->win.x);
@@ -152,7 +277,7 @@ wins_scrollwin_init (scrollwin_t *sw)
 
 /* Free an already created scrollwin. */
 void
-wins_scrollwin_delete (scrollwin_t *sw)
+wins_scrollwin_delete (struct scrollwin *sw)
 {
   EXIT_IF (sw == 0, "null pointer");
   delwin(sw->win.p);
@@ -161,7 +286,7 @@ wins_scrollwin_delete (scrollwin_t *sw)
 
 /* Display a scrolling window. */
 void
-wins_scrollwin_display (scrollwin_t *sw)
+wins_scrollwin_display (struct scrollwin *sw)
 {
   const int visible_lines = sw->win.h - sw->pad.y - 1;
   
@@ -181,24 +306,35 @@ wins_scrollwin_display (scrollwin_t *sw)
   wnoutrefresh (sw->win.p);
   pnoutrefresh (sw->pad.p, sw->first_visible_line, 0, sw->pad.y, sw->pad.x,
                 sw->win.h - sw->pad.y + 1, sw->win.w - sw->win.x);
-  doupdate ();
+  wins_doupdate ();
 }
 
 void
-wins_scrollwin_up (scrollwin_t *sw, int amount)
+wins_scrollwin_up (struct scrollwin *sw, int amount)
 {
   if (sw->first_visible_line > 0)
     sw->first_visible_line -= amount;
 }
 
 void
-wins_scrollwin_down (scrollwin_t *sw, int amount)
+wins_scrollwin_down (struct scrollwin *sw, int amount)
 {
   if (sw->total_lines
       > (sw->first_visible_line + sw->win.h - sw->pad.y - 1))
     sw->first_visible_line += amount;
 }
 
+void
+wins_reinit_panels (void)
+{
+  delwin (win[CAL].p);
+  delwin (win[APP].p);
+  delwin (apad.ptrwin);
+  delwin (win[TOD].p);
+  wins_get_config ();
+  wins_init_panels ();
+}
+
 /* 
  * Delete the existing windows and recreate them with their new
  * size and placement.
@@ -206,11 +342,11 @@ wins_scrollwin_down (scrollwin_t *sw, int amount)
 void
 wins_reinit (void)
 {
-  delwin (win[STA].p);
   delwin (win[CAL].p);
   delwin (win[APP].p);
   delwin (apad.ptrwin);
   delwin (win[TOD].p);
+  delwin (win[STA].p);
   wins_get_config ();
   wins_init ();
   if (notify_bar ())
@@ -264,19 +400,22 @@ wins_get_config (void)
       win[NOT].x = 0;
     }
 
+  win[CAL].w = wins_sbar_width ();
+  win[CAL].h = CALHEIGHT;
+  
   if (layout <= 4)
     {				/* APPOINTMENT is the biggest panel */
-      win[APP].w = col - CALWIDTH;
+      win[APP].w = col - win[CAL].w;
       win[APP].h = row - (win[STA].h + win[NOT].h);
-      win[TOD].w = CALWIDTH;
-      win[TOD].h = row - (CALHEIGHT + win[STA].h + win[NOT].h);
+      win[TOD].w = win[CAL].w;
+      win[TOD].h = row - (win[CAL].h + win[STA].h + win[NOT].h);
     }
   else
     {				/* TODO is the biggest panel */
-      win[TOD].w = col - CALWIDTH;
+      win[TOD].w = col - win[CAL].w;
       win[TOD].h = row - (win[STA].h + win[NOT].h);
-      win[APP].w = CALWIDTH;
-      win[APP].h = row - (CALHEIGHT + win[STA].h + win[NOT].h);
+      win[APP].w = win[CAL].w;
+      win[APP].h = row - (win[CAL].h + win[STA].h + win[NOT].h);
     }
 
   /* defining the layout */
@@ -287,7 +426,7 @@ wins_get_config (void)
       win[APP].x = 0;
       win[CAL].y = 0;
       win[TOD].x = win[APP].w;
-      win[TOD].y = CALHEIGHT;
+      win[TOD].y = win[CAL].h;
       win[CAL].x = win[APP].w;
       break;
     case 2:
@@ -303,22 +442,22 @@ wins_get_config (void)
       win[TOD].x = 0;
       win[CAL].x = 0;
       win[CAL].y = 0;
-      win[APP].x = CALWIDTH;
-      win[TOD].y = CALHEIGHT;
+      win[APP].x = win[CAL].w;
+      win[TOD].y = win[CAL].h;
       break;
     case 4:
       win[APP].y = 0;
       win[TOD].x = 0;
       win[TOD].y = 0;
       win[CAL].x = 0;
-      win[APP].x = CALWIDTH;
+      win[APP].x = win[CAL].w;
       win[CAL].y = win[TOD].h;
       break;
     case 5:
       win[TOD].y = 0;
       win[TOD].x = 0;
       win[CAL].y = 0;
-      win[APP].y = CALHEIGHT;
+      win[APP].y = win[CAL].h;
       win[APP].x = win[TOD].w;
       win[CAL].x = win[TOD].w;
       break;
@@ -335,15 +474,15 @@ wins_get_config (void)
       win[APP].x = 0;
       win[CAL].x = 0;
       win[CAL].y = 0;
-      win[TOD].x = CALWIDTH;
-      win[APP].y = CALHEIGHT;
+      win[TOD].x = win[CAL].w;
+      win[APP].y = win[CAL].h;
       break;
     case 8:
       win[TOD].y = 0;
       win[APP].x = 0;
       win[CAL].x = 0;
       win[APP].y = 0;
-      win[TOD].x = CALWIDTH;
+      win[TOD].x = win[CAL].w;
       win[CAL].y = win[APP].h;
       break;
     }
@@ -404,12 +543,8 @@ border_nocolor (WINDOW *window)
   wnoutrefresh (window);
 }
 
-/* 
- * Update all of the three windows and put a border around the
- * selected window.
- */
 void
-wins_update (void)
+wins_update_border (void)
 {
   switch (slctd_win)
     {
@@ -432,15 +567,30 @@ wins_update (void)
       EXIT (_("no window selected"));
       /* NOTREACHED */
     }
+}
 
+void
+wins_update_panels (void)
+{
   apoint_update_panel (slctd_win);
   todo_update_panel (slctd_win);
-  calendar_update_panel (win[CAL].p);
+  calendar_update_panel (&win[CAL]);
+}
+
+/* 
+ * Update all of the three windows and put a border around the
+ * selected window.
+ */
+void
+wins_update (void)
+{
+  wins_update_border ();
+  wins_update_panels ();
   wins_status_bar ();
   if (notify_bar ())
     notify_update_bar ();
   wmove (win[STA].p, 0, 0);
-  doupdate ();
+  wins_doupdate ();
 }
 
 /* Reset the screen, needed when resizing terminal for example. */
@@ -448,7 +598,7 @@ void
 wins_reset (void)
 {
   endwin ();
-  refresh ();
+  wins_refresh ();
   curs_set (0);
   wins_reinit ();
   wins_update ();
@@ -479,19 +629,19 @@ wins_launch_external (const char *file, const char *cmd)
   endwin ();
   ui_mode = UI_CMDLINE;
   clear ();
-  refresh ();
+  wins_refresh ();
   (void)system (p);
   reset_prog_mode ();
   clearok (curscr, TRUE);
   curs_set (0);
   ui_mode = UI_CURSES;
-  refresh ();
+  wins_refresh ();
   if (notify_bar ())
     notify_start_main_thread ();
   mem_free (p);
 }
 
-#define NB_CAL_CMDS	24	/* number of commands while in cal view */
+#define NB_CAL_CMDS	27	/* number of commands while in cal view */
 #define NB_APP_CMDS	31	/* same thing while in appointment view */
 #define NB_TOD_CMDS	30	/* same thing while in todo view */
 #define TOTAL_CMDS	NB_CAL_CMDS + NB_APP_CMDS + NB_TOD_CMDS
@@ -509,53 +659,54 @@ void
 wins_status_bar (void)
 {
 #define NB_PANELS	3	/* 3 panels: CALENDAR, APPOINTMENT, TODO */
-  window_e which_pan;
+  enum win which_pan;
   int start, end;
   const int pos[NB_PANELS + 1] =
       { 0, NB_CAL_CMDS, NB_CAL_CMDS + NB_APP_CMDS, TOTAL_CMDS };
 
-  binding_t help   = {_("Help"),     KEY_GENERIC_HELP};
-  binding_t quit   = {_("Quit"),     KEY_GENERIC_QUIT};
-  binding_t save   = {_("Save"),     KEY_GENERIC_SAVE};
-  binding_t cut    = {_("Cut"),      KEY_GENERIC_CUT};
-  binding_t paste  = {_("Paste"),    KEY_GENERIC_PASTE};    
-  binding_t chgvu  = {_("Chg View"), KEY_GENERIC_CHANGE_VIEW};
-  binding_t import = {_("Import"),   KEY_GENERIC_IMPORT};  
-  binding_t export = {_("Export"),   KEY_GENERIC_EXPORT};
-  binding_t togo   = {_("Go to"),    KEY_GENERIC_GOTO};
-  binding_t othr   = {_("OtherCmd"), KEY_GENERIC_OTHER_CMD};
-  binding_t conf   = {_("Config"),   KEY_GENERIC_CONFIG_MENU};  
-  binding_t draw   = {_("Redraw"),   KEY_GENERIC_REDRAW};
-  binding_t appt   = {_("Add Appt"), KEY_GENERIC_ADD_APPT};
-  binding_t todo   = {_("Add Todo"), KEY_GENERIC_ADD_TODO};
-  binding_t gnday  = {_("+1 Day"),   KEY_GENERIC_NEXT_DAY};
-  binding_t gpday  = {_("-1 Day"),   KEY_GENERIC_PREV_DAY};
-  binding_t gnweek = {_("+1 Week"),  KEY_GENERIC_NEXT_WEEK};
-  binding_t gpweek = {_("-1 Week"),  KEY_GENERIC_PREV_WEEK};
-  binding_t today  = {_("Today"),    KEY_GENERIC_GOTO_TODAY};
-  binding_t up     = {_("Up"),       KEY_MOVE_UP};
-  binding_t down   = {_("Down"),     KEY_MOVE_DOWN};  
-  binding_t left   = {_("Left"),     KEY_MOVE_LEFT};
-  binding_t right  = {_("Right"),    KEY_MOVE_RIGHT};  
-  binding_t weekb  = {_("beg Week"), KEY_START_OF_WEEK};
-  binding_t weeke  = {_("end Week"), KEY_END_OF_WEEK};
-  binding_t add    = {_("Add Item"), KEY_ADD_ITEM};
-  binding_t del    = {_("Del Item"), KEY_DEL_ITEM};
-  binding_t edit   = {_("Edit Itm"), KEY_EDIT_ITEM};
-  binding_t view   = {_("View"),     KEY_VIEW_ITEM};  
-  binding_t flag   = {_("Flag Itm"), KEY_FLAG_ITEM};
-  binding_t rept   = {_("Repeat"),   KEY_REPEAT_ITEM};
-  binding_t enote  = {_("EditNote"), KEY_EDIT_NOTE};
-  binding_t vnote  = {_("ViewNote"), KEY_VIEW_NOTE};
-  binding_t rprio  = {_("Prio.+"),   KEY_RAISE_PRIORITY};
-  binding_t lprio  = {_("Prio.-"),   KEY_LOWER_PRIORITY};
-
+  struct binding help   = {_("Help"),     KEY_GENERIC_HELP};
+  struct binding quit   = {_("Quit"),     KEY_GENERIC_QUIT};
+  struct binding save   = {_("Save"),     KEY_GENERIC_SAVE};
+  struct binding cut    = {_("Cut"),      KEY_GENERIC_CUT};
+  struct binding paste  = {_("Paste"),    KEY_GENERIC_PASTE};    
+  struct binding chgvu  = {_("Chg Win"), KEY_GENERIC_CHANGE_VIEW};
+  struct binding import = {_("Import"),   KEY_GENERIC_IMPORT};  
+  struct binding export = {_("Export"),   KEY_GENERIC_EXPORT};
+  struct binding togo   = {_("Go to"),    KEY_GENERIC_GOTO};
+  struct binding othr   = {_("OtherCmd"), KEY_GENERIC_OTHER_CMD};
+  struct binding conf   = {_("Config"),   KEY_GENERIC_CONFIG_MENU};  
+  struct binding draw   = {_("Redraw"),   KEY_GENERIC_REDRAW};
+  struct binding appt   = {_("Add Appt"), KEY_GENERIC_ADD_APPT};
+  struct binding todo   = {_("Add Todo"), KEY_GENERIC_ADD_TODO};
+  struct binding gnday  = {_("+1 Day"),   KEY_GENERIC_NEXT_DAY};
+  struct binding gpday  = {_("-1 Day"),   KEY_GENERIC_PREV_DAY};
+  struct binding gnweek = {_("+1 Week"),  KEY_GENERIC_NEXT_WEEK};
+  struct binding gpweek = {_("-1 Week"),  KEY_GENERIC_PREV_WEEK};
+  struct binding today  = {_("Today"),    KEY_GENERIC_GOTO_TODAY};
+  struct binding nview  = {_("Nxt View"), KEY_GENERIC_SCROLL_DOWN};
+  struct binding pview  = {_("Prv View"), KEY_GENERIC_SCROLL_UP};
+  struct binding up     = {_("Up"),       KEY_MOVE_UP};
+  struct binding down   = {_("Down"),     KEY_MOVE_DOWN};  
+  struct binding left   = {_("Left"),     KEY_MOVE_LEFT};
+  struct binding right  = {_("Right"),    KEY_MOVE_RIGHT};  
+  struct binding weekb  = {_("beg Week"), KEY_START_OF_WEEK};
+  struct binding weeke  = {_("end Week"), KEY_END_OF_WEEK};
+  struct binding add    = {_("Add Item"), KEY_ADD_ITEM};
+  struct binding del    = {_("Del Item"), KEY_DEL_ITEM};
+  struct binding edit   = {_("Edit Itm"), KEY_EDIT_ITEM};
+  struct binding view   = {_("View"),     KEY_VIEW_ITEM};  
+  struct binding flag   = {_("Flag Itm"), KEY_FLAG_ITEM};
+  struct binding rept   = {_("Repeat"),   KEY_REPEAT_ITEM};
+  struct binding enote  = {_("EditNote"), KEY_EDIT_NOTE};
+  struct binding vnote  = {_("ViewNote"), KEY_VIEW_NOTE};
+  struct binding rprio  = {_("Prio.+"),   KEY_RAISE_PRIORITY};
+  struct binding lprio  = {_("Prio.-"),   KEY_LOWER_PRIORITY};
   
-  binding_t *binding[TOTAL_CMDS] = {
+  struct binding *binding[TOTAL_CMDS] = {
     /* calendar keys */
-    &help, &quit, &save, &chgvu, &import, &export, &up, &down, &left, &right,
-    &togo, &othr, &weekb, &weeke, &conf, &draw, &appt, &todo, &gnday, &gpday,
-    &gnweek, &gpweek, &today, &othr,
+    &help, &quit, &save, &chgvu, &nview, &pview, &up, &down, &left, &right,
+    &togo, &othr, &import, &export, &weekb, &weeke, &appt, &todo,
+    &gnday, &gpday, &gnweek, &gpweek, &draw, &othr, &today, &conf, &othr,
     /* appointment keys */
     &help, &quit, &save, &chgvu, &import, &export, &add, &del, &edit, &view,
     &draw, &othr, &rept, &flag, &enote, &vnote, &up, &down, &gnday, &gpday,
